diff -r fa384205bbb7 sys/fs/nfsclient/nfs_clvfsops.c
--- a/sys/fs/nfsclient/nfs_clvfsops.c	Wed Oct 09 17:40:21 2013 +0800
+++ b/sys/fs/nfsclient/nfs_clvfsops.c	Tue Oct 15 19:41:47 2013 +0800
@@ -115,6 +115,7 @@
 static vfs_statfs_t nfs_statfs;
 static vfs_sync_t nfs_sync;
 static vfs_sysctl_t nfs_sysctl;
+static vfs_purge_t nfs_purge;
 
 /*
  * nfs vfs operations.
@@ -129,6 +130,7 @@
 	.vfs_uninit =		ncl_uninit,
 	.vfs_unmount =		nfs_unmount,
 	.vfs_sysctl =		nfs_sysctl,
+       .vfs_purge =         nfs_purge,
 };
 VFS_SET(nfs_vfsops, nfs, VFCF_NETWORK);
 
@@ -1602,8 +1604,22 @@
 }
 
 /*
+ * Purge any RPCs in progress, so that they will all return errors.
+ * This allows dounmount() to continue as far as VFS_UNMOUNT() for a
+ * forced dismount.
+ */
+static void
+nfs_purge(struct mount *mp)
+{
+	struct nfsmount *nmp = VFSTONFS(mp);
+
+	newnfs_nmcancelreqs(nmp);
+}
+
+/*
  * Extract the information needed by the nlm from the nfs vnode.
  */
+
 static void
 nfs_getnlminfo(struct vnode *vp, uint8_t *fhp, size_t *fhlenp,
     struct sockaddr_storage *sp, int *is_v3p, off_t *sizep,
diff -r fa384205bbb7 sys/kern/vfs_mount.c
--- a/sys/kern/vfs_mount.c	Wed Oct 09 17:40:21 2013 +0800
+++ b/sys/kern/vfs_mount.c	Tue Oct 15 19:41:47 2013 +0800
@@ -1268,8 +1268,16 @@
 	}
 	mp->mnt_kern_flag |= MNTK_UNMOUNT | MNTK_NOINSMNTQ;
 	/* Allow filesystems to detect that a forced unmount is in progress. */
-	if (flags & MNT_FORCE)
+	if (flags & MNT_FORCE) {
 		mp->mnt_kern_flag |= MNTK_UNMOUNTF;
+		MNT_IUNLOCK(mp);
+		/*
+		 * Must be done after setting MNTK_UNMOUNTF and before
+		 * waiting for mnt_lockref to become 0.
+		 */
+		VFS_PURGE(mp);
+		MNT_ILOCK(mp);
+	}
 	error = 0;
 	if (mp->mnt_lockref) {
 		mp->mnt_kern_flag |= MNTK_DRAINING;
diff -r fa384205bbb7 sys/sys/mount.h
--- a/sys/sys/mount.h	Wed Oct 09 17:40:21 2013 +0800
+++ b/sys/sys/mount.h	Tue Oct 15 19:41:47 2013 +0800
@@ -634,6 +634,7 @@
 typedef int vfs_sysctl_t(struct mount *mp, fsctlop_t op,
 		    struct sysctl_req *req);
 typedef void vfs_susp_clean_t(struct mount *mp);
+typedef void vfs_purge_t(struct mount *mp);
 
 struct vfsops {
 	vfs_mount_t		*vfs_mount;
@@ -651,6 +652,7 @@
 	vfs_extattrctl_t	*vfs_extattrctl;
 	vfs_sysctl_t		*vfs_sysctl;
 	vfs_susp_clean_t	*vfs_susp_clean;
+	vfs_purge_t		*vfs_purge;
 };
 
 vfs_statfs_t	__vfs_statfs;
@@ -712,6 +714,13 @@
 		mtx_assert(&Giant, MA_OWNED);				\
 } while (0)
 
+#define VFS_PURGE(MP) do {                                              \
+	if (*(MP)->mnt_op->vfs_purge != NULL) {                         \
+		(*(MP)->mnt_op->vfs_purge)(MP);                         \
+	}                                                               \
+} while (0)
+
+
 #define VFS_KNOTE_LOCKED(vp, hint) do					\
 {									\
 	if (((vp)->v_vflag & VV_NOKNOTE) == 0)				\
